#!/usr/bin/python3
'''
file sispmctl.py - controlling EnerGenie EG-PMS

Copyright (c) 2016, Heinrich Schuchardt <xypron.glpk@gmx.de>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''

import getopt
import sispm
import sys

def checkport(dev, p):
	"""
	Checks if port p exists on the device.

	@param dev device
	@param p port
	@return port exists
	"""
	pmin = sispm.getminport(dev)
	pmax = sispm.getmaxport(dev)
	if p < pmin or p > pmax:
		print("Device {} only has ports {}..{}".format(
			sispm.getid(dev), pmin, pmax))
		return False
	return True

def main():
	# Find our devices.
	devices = sispm.connect()

	# Were they found?
	if len(devices) == 0:
		print('No device found')
		sys.exit(1)

	# If there is only one device, use it as default.
	if len(devices) == 1:
		dev = devices[0]
	else:
		dev = None

	# Define command line options.
	try:
		opts, args = getopt.getopt(sys.argv[1:], "D:d:f:ho:t:")
	except getopt.GetoptError as err:
		print(str(err))
		usage()
		sys.exit(2)

	# Handle command line.
	for o, a in opts:
		if o == "-D":
			dev = None
			for d in devices:
				if sispm.getid(d) == a:
					dev = d
					break
			if dev == None:
				print("device with id {} not found".format(a))
				break
		elif o == "-d":
			d = int(a)
			if d < 0 or d >= len(devices):
				print("unknown device {}".format(d))
				break
			dev = devices[d];
		elif o == "-f":
			p = int(a)
			if not checkport(dev, p):
				break
			sispm.switchoff(dev, p)
		elif o == "-h":
			usage()
			print()
		elif o == "-o":
			p = int(a)
			if not checkport(dev, p):
				break
			sispm.switchon(dev, p)
		elif o == "-t":
			p = int(a)
			if not checkport(dev, p):
				break
			if sispm.getstatus(dev, p) == 0:
				sispm.switchon(dev, p)
			else:
				sispm.switchoff(dev, p)
		else:
			break

	# Always output the device status.
	status(devices)

	# Workaround for bug in old version of usb library.
	devices = None

def status(devices):
	"""
	Outputs the status of all devices.

	@param devices list of devices
	"""
	for dev in devices:
		print('device {}'.format(devices.index(dev)), end=", ")
		# Print device id.
		print(sispm.getid(dev))
		# Print status of all outlets.
		for i in range(sispm.getminport(dev), 1 + sispm.getmaxport(dev)):
			print('\tstatus[{}] = {}'.format(i, sispm.getstatus(dev, i)))

def usage():
	"""
	Outputs the online help.
	"""
	print("Usage: sispmctl.py [OPTIONS]")
	print("Switches Enermax USB controlled plugstrips")
	print()
	print("  -D ID         id of device to be controlled")
	print("  -d DEVICE     index of device to be controlled")
	print("  -f OUTLET     switch outlet off")
	print("  -h            print this help")
	print("  -o OUTLET     switch outlet on")
	print("  -t OUTLET     toggle outlet")

if __name__ == "__main__":
	main()




#!/usr/bin/env python
#
# Copyright (c) 2016, Heinrich Schuchardt <xypron.glpk@gmx.de>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Controling EnerGenie EG-PMS

A library to control the EnerGenie multi-outlet power strips.

The EG-PMS has an USB interface. Four outlets (numbered 1 - 4) can be switched
on and off via USB.

The library depends on PyUSB (https://github.com/walac/pyusb).

Per default, only root is allowed to use devices directly,
therefore the library also only works as root. 

To allow group sispmctl access copy file /lib/udev/rules.d/60-sispmctl.rules
with the following content

SUBSYSTEM=="usb", ATTR{idVendor}=="04b4", ATTR{idProduct}=="fd10", GROUP="sispmctl", MODE="660"
SUBSYSTEM=="usb", ATTR{idVendor}=="04b4", ATTR{idProduct}=="fd11", GROUP="sispmctl", MODE="660"
SUBSYSTEM=="usb", ATTR{idVendor}=="04b4", ATTR{idProduct}=="fd12", GROUP="sispmctl", MODE="660"
SUBSYSTEM=="usb", ATTR{idVendor}=="04b4", ATTR{idProduct}=="fd13", GROUP="sispmctl", MODE="660"
SUBSYSTEM=="usb", ATTR{idVendor}=="04b4", ATTR{idProduct}=="fd15", GROUP="sispmctl", MODE="660"

Then reload the udev rules with

udevadm control --reload-rules

SisPM is licensed under a modified BSD license.
"""

import usb.core
import usb.util

def connect():
	"""
	Returns the list of compatible devices.

	@return: device list
	"""
	ret = list()
	ret += list(usb.core.find(find_all=True,
		idVendor=0x04b4, idProduct=0xfd10))
	ret += list(usb.core.find(find_all=True,
		idVendor=0x04b4, idProduct=0xfd11))
	ret += list(usb.core.find(find_all=True,
		idVendor=0x04b4, idProduct=0xfd12))
	ret += list(usb.core.find(find_all=True,
		idVendor=0x04b4, idProduct=0xfd13))
	ret += list(usb.core.find(find_all=True,
		idVendor=0x04b4, idProduct=0xfd15))
	return ret

def getid(dev):
	"""
	Gets the id of a device.

	@return: id
	"""
	buf = bytes([0x00, 0x00, 0x00, 0x00, 0x00]);
	id =  dev.ctrl_transfer(0xa1, 0x01, 0x0301, 0, buf, 500)
	if (len(id) == 0):
		return None
	ret = ''
	sep = ''
	for x in id:
		ret += sep
		ret += format(x, '02x')
		sep = ':'
	return ret

def getmaxport(dev):
	"""
	Gets the maximum outlet number of a device.

	@return: maximum outlet number
	"""
	assert dev.idVendor == 0x04b4

	if dev.idProduct == 0xfd10:
		return 0
	if dev.idProduct == 0xfd11:
		return 1
	return 4

def getminport(dev):
	"""
	Gets the minimum outlet number of a device.

	@return: minimum outlet number
	"""

	assert dev.idVendor == 0x04b4

	if dev.idProduct == 0xfd10:
		return 0
	return 1

def getstatus(dev, i):
	"""
	Gets the status of outlet i of the device.

	@param dev: device
	@param i: outlet
	@return: status
	"""
	assert i >= getminport(dev) and i <= getmaxport(dev)
	buf = bytes([3 * i, 0x03, 0x00, 0x00, 0x00]);
	buf = dev.ctrl_transfer(0xa1, 0x01, 0x0300 + 3 * i, 0, buf, 500)
	return 1 & buf[1]

def switchoff(dev, i):
	"""
	Switches outlet i of the device off.

	@param dev: device
	@param i: outlet
	"""
	assert i >= getminport(dev) and i <= getmaxport(dev)
	buf = bytes([3 * i, 0x00, 0x00, 0x00, 0x00]);
	buf = dev.ctrl_transfer(0x21, 0x09, 0x0300 + 3 * i, 0, buf, 500)

def switchon(dev, i):
	"""
	Switches outlet i of the device on.

	@param dev: device
	@param i: outlet
	"""
	assert i >= getminport(dev) and i <= getmaxport(dev)
	buf = bytes([3 * i, 0x03, 0x00, 0x00, 0x00]);
	buf = dev.ctrl_transfer(0x21, 0x09, 0x0300 + 3 * i, 0, buf, 500)




#!/usr/bin/python3
'''
file printenv.py - print U-Boot environment

Copyright (c) 2016, Heinrich Schuchardt <xypron.glpk@gmx.de>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''

"""
This example demonstrates how to access U-Boot when a computer is being
switched on.

After toggling power off and on the script waits for serial output from
U-Boot to appear. It issues two CRs to enter U-Boot. The printenv command is
used to print the environment variables. At last the computer is switched off
again.

Please, adjust device id, outlet number and serial port as required.
"""

import serial
import sispm
import time

id = '01:01:53:50:26'
port = 1
tty = '/dev/ttyUSB0'

def skip_output(ser, test = None):
	"""
	Reads serial input until a test string is reached.

	@param ser: serial connection
	@param test: string to test for
	"""
	
	line = ''
	while True:
		c = ser.read(1)
		if (len(c) == 0):
			break
		if (c[0] == 0x0d):
			print(line)
			line = ''
		elif (c[0] >= 32 and c[0] < 128):
			line += chr(c[0])
			if (test and line.find(test) != -1):
				break
	print(line)


# Find Enermax devices.
devices = sispm.connect()

# Were they found?
if len(devices) == 0:
	print('No device found')
	quit()

# Find the device with the correct id
dev = None
for d in devices:
	if sispm.getid(d) == id:
		dev = d
		break

if dev is None:
	print('Device ' + id + ' not found')
	quit()

# Open serial connection
ser = serial.Serial(
	port = tty, \
	baudrate =  115200, \
	stopbits = serial.STOPBITS_ONE, \
	bytesize = serial.EIGHTBITS, \
	timeout = 5)

# Switch device off
print("== Switching off ==")
sispm.switchoff(dev, port)

# Wait for two seconds
time.sleep(2)

# Switch device on
print("== Switching on ==")
sispm.switchon(dev, port)

# Wait for U-Boot message
skip_output(ser, 'stop autoboot:')

# Stop boot process
ser.write(b'\r\r')
skip_output(ser)

# Print environment
ser.write(b'printenv\n')
skip_output(ser)

# Switch device off
print("== Switching off ==")
sispm.switchoff(dev, port)

# Close serial connection
ser.close()




#!/usr/bin/env python3
#
# Copyright (c) 2016, Heinrich Schuchardt <xypron.glpk@gmx.de>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from distutils.core import setup
import setuptools

import sispm

setup(
	name='pysispm',
	version='0.6',
	description='Library for EnerGenie USB controlled powerstrips',
	author='Heinrich Schuchardt',
	author_email='xypron.glpk@gmx.de',
	license = 'BSD',
	url='https://github.com/xypron/pysispm',
	packages=['sispm'],
	install_requires=["pyusb >= 1.0.0a"],
	long_description =
"""
PySisPM is a library to control the EnerGenie EG-PMS and EG-PM2 multiple socket
powerstrips. The outlets can be switched on and off via USB.

The library depends on PyUSB (https://github.com/walac/pyusb).

Per default, only root is allowed to use devices directly,
therefore the library also only works as root.

To allow group sispmctl access copy file /lib/udev/rules.d/60-sispmctl.rules
with the following content::

    SUBSYSTEM=="usb", ATTR{idVendor}=="04b4", ATTR{idProduct}=="fd10", GROUP="sispmctl", MODE="660"
    SUBSYSTEM=="usb", ATTR{idVendor}=="04b4", ATTR{idProduct}=="fd11", GROUP="sispmctl", MODE="660"
    SUBSYSTEM=="usb", ATTR{idVendor}=="04b4", ATTR{idProduct}=="fd12", GROUP="sispmctl", MODE="660"
    SUBSYSTEM=="usb", ATTR{idVendor}=="04b4", ATTR{idProduct}=="fd13", GROUP="sispmctl", MODE="660"
    SUBSYSTEM=="usb", ATTR{idVendor}=="04b4", ATTR{idProduct}=="fd15", GROUP="sispmctl", MODE="660"

Then reload the udev rules with::

    udevadm control --reload-rules
""",
	classifiers=[
		'Development Status :: 4 - Beta',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: BSD License',
		'Operating System :: POSIX',
		'Programming Language :: Python :: 3',
		'Topic :: System :: Hardware :: Hardware Drivers'
	]
)
